/***********************************************************************
 *         
 *      H E L I C O P T E R   C O L L E C T I V E   L E V E R 
 *         
 *         
 *         Using ArduinoJoystickLibrary by Matthew Heironimus
 *        https://github.com/MHeironimus/ArduinoJoystickLibrary
 **********************************************************************/
#include "Joystick.h"
/*--------------------------------------------------------
  "THE BEER-WARE LICENSE" (Revision 42):
  Alex Kostyuk wrote this code. As long as you retain this 
  notice you can do whatever you want with this stuff. 
  If we meet some day, and you think this stuff is worth it, 
  you can buy me a beer in return.
----------------------------------------------------------*/
/*
Joystick_::Joystick_(
  uint8_t hidReportId,
  uint8_t joystickType,
    uint8_t buttonCount,
  uint8_t hatSwitchCount,
  bool includeXAxis,
  bool includeYAxis,
  bool includeZAxis,
  bool includeRxAxis,
  bool includeRyAxis,
  bool includeRzAxis,
  bool includeRudder,
  bool includeThrottle,
  bool includeAccelerator,
  bool includeBrake,
  bool includeSteering)
*/

#define ANALOG_FIFO_SIZE (10)
#define CHANNELS_NUMBER  (2)

typedef struct
{
  int   push_indx;
  int   pop_indx;
  long buff[ANALOG_FIFO_SIZE];
} SIMPLE_FIFO;

SIMPLE_FIFO analog_fifo[CHANNELS_NUMBER];

short fifo_push(SIMPLE_FIFO *fifo, short data);
short fifo_pop(SIMPLE_FIFO *fifo, short *data);

void moving_average_filter_init(void);
short moving_average_filter(int ch, short in);

Joystick_ Joystick(JOYSTICK_DEFAULT_REPORT_ID, 
  JOYSTICK_TYPE_MULTI_AXIS, 0, 0,
  false, false, true, false, false, false,
  false, true, false, false, false);


void setup()
{
    moving_average_filter_init();
    Joystick.begin(false);
}

void loop(void)
{
    static unsigned long ms_reftime = 0;
    int cur;
    unsigned long cur_time = millis();
    static int integrator_throttle = 0;//, integrator_z_axis = 0;
    
    if((cur_time - ms_reftime) >= 10) 
    {
       ms_reftime = cur_time;

              cur = analogRead(A1);
              Joystick.setZAxis(moving_average_filter(0, cur));
              

              cur = analogRead(A0);
          #ifdef MI_2_X_PLANE  
              cur = moving_average_filter(1, cur);
              integrator_throttle += ((cur - integrator_throttle)/16);
              Joystick.setThrottle(integrator_throttle);
          #else
              Joystick.setThrottle(moving_average_filter(1, cur));
          #endif
        Joystick.sendState();
    }
}


void moving_average_filter_init(void)
{
   memset(analog_fifo, 0, sizeof(analog_fifo));
}

short moving_average_filter(int ch, short in)
{
static short sum[CHANNELS_NUMBER] = {0,0};
short tmp = in;
short prev;
     sum[ch] += in;
     if(!fifo_push(&analog_fifo[ch], in))
     {
       fifo_pop(&analog_fifo[ch], &prev);
       fifo_push(&analog_fifo[ch], in);
       sum[ch] -= prev;
       tmp = (sum[ch] / (ANALOG_FIFO_SIZE-1));
       
   
       return tmp;
     } 
  
         
return 0;
}

short fifo_push(SIMPLE_FIFO *fifo, short data)
{
    int indx;
    indx = fifo->push_indx + 1;
    indx = indx % ANALOG_FIFO_SIZE;

    if (indx == fifo->pop_indx)
        return 0; // fifo is full
    fifo->buff[indx] = data;
    fifo->push_indx = indx;
    return 1;
}

short fifo_pop(SIMPLE_FIFO *fifo, short *data)
{
    int indx;
    if (fifo->push_indx == fifo->pop_indx)
        return 0; // fifo is empty
    indx = fifo->pop_indx + 1;
    indx = indx % ANALOG_FIFO_SIZE;
    *data = fifo->buff[indx];
    fifo->pop_indx = indx;
    return 1;
}

